<?php 
namespace DoThatTask\Config;
/**
 * Locales Constants
 *
 * @package DoThatTask/Config
 * 
 */

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'Constant' ) ) {

	final class Constant
	{
		protected static $_pluginData;
		/**
		 * Main DoThatTask Instance.
		 *
		 * @since 1.1.5
		 * @static
		 * @see init()
		 * @return DoThatTask - Main instance.
		 */
		public static function init() {

			/**
			 *
			 * BASE DIR constant
			 *
			 */
			define( 'DTT_BASE' , dirname(plugin_basename( dirname(__FILE__) ) ) );
			define( 'DTT_PATH' , plugin_dir_path( dirname(__FILE__) ) );
			define( 'DTT_URL' , plugin_dir_url( dirname(__FILE__) ) );

			/**
			 *
			 * Basic Info Get From Plugin Main File.
			 *
			 */
			self::$_pluginData = get_file_data( DTT_PATH.'index.php' , array( 'name'=>'Plugin Name', 'version'=>'Version', 'text'=>'Description','mode'=>'Mode','perfix'=>'Perfix' ,'text_domain'=>'Text Domain','powered_by'=>'Author URI','protocol'=>'Protocol','app_token'=>'APP Token','slug'=>'Slug','storage'=>'Storage') ,false);

			define( 'DTT_PREFIX',  self::$_pluginData['perfix'].'_');
			define( 'DTT_DOMAIN', $_SERVER['HTTP_HOST']);
			define( 'DTT_CURRENT_URL', (stripos($_SERVER['SERVER_PROTOCOL'],'https') === true ? 'https://' : 'https://') . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
			/**
			 *
			 * API constant define
			 *
			 */
			if ( ! defined( 'DTT_API_MODE' ) ) {
				define( 'DTT_API_MODE' ,strtoupper(self::$_pluginData['mode']));
			}

			if ( strtolower(self::$_pluginData['mode']) == "live" ) {
				define( 'DTT_API_URL', (self::$_pluginData['protocol'] === 'https' ? 'https' : 'http' )."://app.dothattask.com/" );	
			} else {
				define( 'DTT_API_URL', "http://localhost:8000/" );
			}
			define( 'DTT_APP_DOMAIN', 'dothattask.com');
			
			self::defineConstant();
			self::getNewVersion();
			define( 'DTT_UPGRADE_LINK', admin_url('plugin-install.php?tab=plugin-information&plugin='.DTT_SLUG));

			if ( strtolower(self::$_pluginData['storage']) == "s3" ) {
				define( 'DTT_ASSETS_URL' , DTT_S3_MEDIA );
				define( 'DTT_UPLOADED_ASSETS_URL' , DTT_S3_MEDIA );
			} else {
				define( 'DTT_ASSETS_URL' , DTT_API_URL );
				define( 'DTT_UPLOADED_ASSETS_URL' , DTT_API_URL ."storage/" );
			}
			if ( strtolower(self::$_pluginData['mode']) == "live" ) {
				define( 'DTT_SNIPPET_URL', DTT_S3_MEDIA .'snippet/' );
			} else {
				define( 'DTT_SNIPPET_URL', "//localhost/dtt-snippet/" );
			}
		}
		protected static function defineConstant(){
			/**
			 * setting page/ extra constant
			 */
			$_extraConstant = [	'NAME'						=> 	self::$_pluginData['name'] ,
								'VERSION'					=> 	self::$_pluginData['version'] ,
								'TEXT'						=> 	self::$_pluginData['text'] ,
								'TEXT_DOMAIN'				=> 	self::$_pluginData['text_domain'],
								'POWERED_BY_URI'			=> 	self::$_pluginData['powered_by'],
								'APP_TOKEN'					=> 	self::$_pluginData['app_token'],
								'SLUG'						=>	self::$_pluginData['slug'],
								'S3_BUCKET'					=>	'https://d1bkvm5f4uej65.cloudfront.net/',
								'S3_MEDIA'					=>	'https://s3.us-east-1.amazonaws.com/media.dothattask.com/',
								'ASSETS'					=> 	DTT_URL.'assets/',
								'TEMPLATE'					=> 	DTT_PATH.'template/',
								'VUE'						=> 	DTT_PATH.'vue/',
								'VUE_COMPONENT'				=> 	DTT_PATH.'vue/components/',
								'VUE_COMMON_COMPONENT'		=> 	DTT_PATH.'vue/components/common',
								'SETTING'					=>	'dtt_settings',
								'PAYPAL'					=>	'dtt_paypal_details',
								'DO_INSTALLATION'			=>	"dtt_plugin_do_install",
								'DO_ACTIVATION_REDIRECT'	=>	"dtt_plugin_do_activation_redirect",
								'ACTIVATION_ERROR_TYPE' 	=>	"dtt_plugin_activation_error_type",
								'ACTIVATION_ERROR_DATA'		=>	"dtt_plugin_activation_error_data",
								'PLUGIN_PRIMARY_EMAIL'		=>	"dtt_plugin_primary_email",
								'PLUGIN_EMAIL_VERIFY_CODE'	=>	"dtt_plugin_email_verify_code",
								'PLUGIN_ACTIVATE'			=>	"dtt_plugin_activate",
								'PLUGIN_UPGRADE'			=>	"dtt_plugin_upgrade",
								'PLUGIN_INIT'				=>	"dtt_plugin_init",
								'PLUGIN_MINIMUM_VERSION'	=>	"dtt_plugin_minimum_version",
								'PLUGIN_USER_ROLE_SLUG'		=>	"dtt_plugin_user_role",
								'PLUGIN_USER_ROLE_NAME'		=>	"DoThatTask User Role",
								'COLOR_PRESET'				=>  [ 'red'=>'#FC4242','pink' => '#FF2DC6',
																				'lightgreen' => '#06AF8F','orange'=>'#F7AC08','yellow'=>'#E3F705','green'=>'#1B9D2C','black'=>'#000000','blue'=>'#425CFB ','purple'=>'#632289','lightgray'=>'#A4A4A4','white'=>'#FFFFFF','charcoal'=>'#444444'
																],
								'DEFAULT_COLOR_PRESET'		=>  'red',
								'TOKEN'						=>	'dtt_token',
							];
			
			foreach($_extraConstant as $con_name=>$con_value){
				self::allmetatags_constants( $con_name, $con_value);
			}
		}
		/**
		 * define constant
		 *
		 */
		protected static function allmetatags_constants( $constant_name, $value ) {
		    $constant_name = 'DTT_' . $constant_name;
		    if ( !defined( $constant_name ) )
		        define( $constant_name, $value );
		}
		/**
		 * define new latest version of plugin
		 *
		 */
		protected static function getNewVersion(){
			$file = DTT_SLUG.'/index.php';
			$current = get_site_transient( 'update_plugins' );
			$response = isset($current->response[ $file ])?$current->response[ $file ]:(isset($current->no_update[ $file ])?$current->no_update[ $file ]:[]);
			if(!empty($response)){
				self::allmetatags_constants( 'LATEST_VERSION', $response->new_version);
			}
		}
	}
	//Call Constant init function on page load
	//Constant::init();
}