<?php
namespace DoThatTask\Includes;

if ( ! defined( 'ABSPATH' ) ) {
	die("You can't access this file directly"); // disable direct access
} 
use DoThatTask\Config\Constant;
use DoThatTask\Includes\Common_Functions;
use DoThatTask\Includes\Dtt_Ajax;

if ( ! class_exists( 'DoThatTask' ) ) {

	final class DoThatTask
	{		
		protected static $_instance = null;
		protected $__userCapability = [ 'read' => true, 'manage_options'=>true ];
		public $_doThatTaskIcon = "";
		public $_unopenTask = 0;
		public $functionClass;
		protected $_pluginAllow = true;
		public $_page = array('dtt_task_manager_settings','dtt_task_manager_view_task','dtt_task_manager_transactions','dtt_task_manager_board','dtt_task_intro');
		public $_jsCss = array('dtt_plugin_details','dtt_task_manager_settings','dtt_task_manager_view_task','dtt_task_manager_transactions','dtt_task_manager_board','dtt_plugin_upgrade','dtt_task_intro');

		/**
		 * Main DoThatTask Instance.
		 *
		 * @since 
		 * @static
		 * @see init()
		 * @return DoThatTask - Main instance.
		 */
		public static function init() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		public function __construct()
		{
			$this->define_constant();
			//This funcation call at init action
			$this->includes();
			//This funcation init all hooks which required for plugin.
			$this->init_hooks();
		}
		protected function define_constant(){
			//Call Constant init function on page load
			Constant::init();
		}
		/**
	     * Add Warning Message for across admin panel
	     * 
	     * when client do not activation plugin then that warning message will appear on every admin pages
	     *
	     * @access  public
	     * @param   none 
	     * @return  string
	     */
		public function dtt_warning_notice()
		{
		    echo '<div class="notice notice-error is-dismissible"><br/>
		    		<b>'.__('DoThatTask Verification required',DTT_TEXT_DOMAIN).'.</b><br/>
		            <p>'.__('You need to verify your email with this domain. Click on verify button to start using Best benefits of this plugin',DTT_TEXT_DOMAIN).'.</p>
		            <a href="admin.php?page=dtt_plugin_details" class="button button-primary">'.__('VERIFY',DTT_TEXT_DOMAIN).'.</a>
		            <br/><br/>
		         </div>';
		}
		/**
	     * Plugin Init.
	     * 
	     * Used to init all resources for plugin use.
	     *
	     * @access  public
	     * @param   none 
	     * @return  none
	     */
		protected function includes()
		{	
			global $dttTaskFunObj;
			ob_start();
		    if(!session_id()) {
		        session_start();
		    }

			$this->functionClass = $dttTaskFunObj = new Common_Functions;
			$this->_doThatTaskIcon = $dttTaskFunObj->__iconUrl;
			
			/* DoThatTask Audit Token */
			if(isset($_REQUEST['dtttoken'])){
				$_SESSION['dtt_audit_token'] = $_REQUEST['dtttoken'];	
				$_SESSION['dtt_audit_token_expire'] = strtotime("+7 day");	
			}else{
				$this->dtt_audit_session();
			}
			if(isset($_SESSION['dtt_audit_token'])){
				$dttTaskFunObj->_auditToken = $_SESSION['dtt_audit_token'];
			}
			if($this->checkTokenExistOrNot()){
				$dttTaskFunObj->_addTaskBtnForAudit = $this->functionClass->_addTaskBtnForAudit = true;
			}

			if($this->functionClass->wpDefaultValues()['user_role'] == DTT_PLUGIN_USER_ROLE_SLUG){
				$this->_pluginAllow = false;
			}

			//Check upgrade is available or not
			$version = sanitize_text_field(isset($_REQUEST['version'])) ? sanitize_text_field($_REQUEST['version']) : get_option(DTT_PLUGIN_MINIMUM_VERSION,'');
			if(!empty($version) && $version > DTT_VERSION){
				if (FALSE === get_option(DTT_PLUGIN_UPGRADE) && FALSE === update_option(DTT_PLUGIN_UPGRADE,FALSE)){
					update_option(DTT_PLUGIN_MINIMUM_VERSION,$version);
					add_option(DTT_PLUGIN_UPGRADE, true);
				}	
			}else{
				if(get_option(DTT_PLUGIN_UPGRADE, false)){
					$this->functionClass->deleteOption([DTT_PLUGIN_UPGRADE]);
				}
			}
			$this->checkStableVersion();
		}
		//Check plugin version is stable or not , if not then redirect on upgrade page.
		private function checkStableVersion(){
			$page = sanitize_text_field(isset($_REQUEST['page'])) ? sanitize_text_field($_REQUEST['page']) : '';
			if(!empty($page)){
				if(in_array($page,$this->_page)){
					$this->functionClass->getSettings();
					if($this->functionClass->_settings['stable_version'] > DTT_VERSION){
						update_option(DTT_PLUGIN_MINIMUM_VERSION,$this->functionClass->_settings['stable_version']);
						exit( wp_redirect('admin.php?page=dtt_plugin_upgrade&version='.$this->functionClass->_settings['stable_version']));
					}
				}elseif(!empty($page) && $page == "dtt_plugin_upgrade"){
					$version = sanitize_text_field(isset($_REQUEST['version'])) ? sanitize_text_field($_REQUEST['version']) : '';
					if(empty($version)){
						$this->functionClass->getSettings();
						update_option(DTT_PLUGIN_MINIMUM_VERSION,$this->functionClass->_settings['stable_version']);
						exit( wp_redirect('admin.php?page=dtt_plugin_upgrade&version='.$this->functionClass->_settings['stable_version']));
					}
				}
			}
		}

		protected function init_hooks(){
			
			$this->aParams = array();	
			
			//Plugin instant redirect on verify email page
			if (!get_option(DTT_DO_INSTALLATION, false) || !get_option(DTT_PLUGIN_ACTIVATE,false)) {
				add_action('admin_init', array($this, 'dtt_plugin_redirect'));
			}
			//Add action for to add warning message across admin panel
			if (get_option(DTT_DO_INSTALLATION, false) && !get_option(DTT_PLUGIN_UPGRADE, false)) {
				$page = sanitize_text_field(isset($_REQUEST['page'])) ? sanitize_text_field($_REQUEST['page']) : '';
				if(empty($page) || (!empty($page) && !in_array($page,$this->_page))){
					add_action( 'admin_notices', array(&$this,'dtt_warning_notice') );
				}
			}
			//Admin menu after activation 
			add_action( 'admin_menu', array(&$this, 'dtt_task_manager_admin_menu'));
			
			//At Front end, add footer  
			add_action( 'wp_footer',array(&$this, 'dtt_task_manager_footer'));
			//At Admin end, add footer 
			add_action( 'admin_footer',array(&$this, 'dtt_task_manager_admin_footer'));
			//Add Scripts with this funcation at front section.
			add_action( 'wp_enqueue_scripts',array(&$this, 'dtt_enqueue_scripts'));
			
			//Add Scripts with this funcation at admin panel 
			add_action( 'admin_enqueue_scripts', array(&$this,'dtt_admin_enqueue_scripts'));
			
			//Ajax Hooks Define in this file
			Dtt_Ajax::init();

			//Activation , deactivation and uninstall hook
			register_activation_hook( DTT_PLUGIN_FILE , array(&$this,'activation_dtt_plugin') );
			register_deactivation_hook( DTT_PLUGIN_FILE , array(&$this,'deactivation_dtt_plugin'));
			register_uninstall_hook( DTT_PLUGIN_FILE , 'uninstall_dtt_plugin');
		}
		function dtt_audit_session() {
			global $dttTaskFunObj;
		    if (isset($_SESSION['dtt_audit_token_expire']) && ($_SESSION['dtt_audit_token_expire'] < time())) {
		        if(isset($_SESSION['dtt_audit_token'])){
					$dttTaskFunObj->_auditToken =  $this->functionClass->_auditToken = $_SESSION['dtt_audit_token'] = '';
					$dttTaskFunObj->_addTaskBtnForAudit = $this->functionClass->_addTaskBtnForAudit = false;
					unset($_SESSION['dtt_audit_token']);
					unset($_SESSION['dtt_audit_token_expire']);
				}
		    }
		}
		/**
	     * Plugin Menu.
	     * 
	     * Used to add menu in admin panel for plugin.
	     *
	     * @access  public
	     * @param   none 
	     * @return  none
	     */
		public function dtt_task_manager_admin_menu()
		{	
			$this->_unopenTask = 0;
			if($this->_pluginAllow){
				/*if (get_option(DTT_PLUGIN_ACTIVATE,false)) {
					$data = $this->functionClass->wpDefaultValues();

					$task_id = sanitize_text_field(isset($_REQUEST['task_id'])) ? sanitize_text_field($_REQUEST['task_id']) : 0;
					$this->_unopenTask = $this->functionClass->getUnOpnedTaskCount($data['admin_email'],intval(!empty($task_id) ? $task_id : 0));
				}*/
				$page = sanitize_text_field(isset($_REQUEST['page'])) ? sanitize_text_field($_REQUEST['page']) : '';
				$aPages = $this->_page;
				if (get_option(DTT_PLUGIN_UPGRADE, false)) {

					add_menu_page(__( 'DoThatTask', DTT_TEXT_DOMAIN ), sprintf(__( 'DoThatTask %s', DTT_TEXT_DOMAIN ),__('Upgrade', DTT_TEXT_DOMAIN )), 'manage_options', 'dtt_plugin_upgrade',array($this, 'dtt_plugin_upgrade'),$this->_doThatTaskIcon);

					$aPages[] = 'dtt_plugin_details';
					if(!empty($page) && in_array($page,$aPages)){
						exit( wp_redirect('admin.php?page=dtt_plugin_upgrade') );
					}
				}elseif (get_option(DTT_DO_INSTALLATION, false)) {

					add_menu_page(__( 'DoThatTask', DTT_TEXT_DOMAIN ), sprintf(__( 'DoThatTask %s', DTT_TEXT_DOMAIN ),__('Installation', DTT_TEXT_DOMAIN )), 'manage_options', 'dtt_plugin_details',array($this, 'dtt_plugin_details'),$this->_doThatTaskIcon);
				
					$aPages[] = 'dtt_plugin_upgrade';
					if(!empty($page) && in_array($page,$aPages)){
						exit( wp_redirect('admin.php?page=dtt_plugin_details') );
					}
				} else{

					add_menu_page(__( 'DoThatTask', DTT_TEXT_DOMAIN ), sprintf(__( 'DoThatTask %s', DTT_TEXT_DOMAIN ),($this->_unopenTask>0?'<span class="update-plugins count-4"><span class="plugin-count">'.$this->_unopenTask.'</span></span>':'')), 'manage_options', 'dtt_task_manager_view_task',array($this, 'dtt_task_manager_view_task'),$this->_doThatTaskIcon);

					add_submenu_page('dtt_task_manager_view_task', __( 'Task Central', DTT_TEXT_DOMAIN ),__( 'Task Central', DTT_TEXT_DOMAIN ), 'manage_options', 'dtt_task_manager_view_task',array($this, 'dtt_task_manager_view_task'));
					
					add_submenu_page('dtt_task_manager_view_task', __( 'My Account', DTT_TEXT_DOMAIN ),__( 'My Account', DTT_TEXT_DOMAIN ), 'manage_options', 'dtt_task_manager_settings',array($this, 'dtt_task_manager_settings'));
				
					add_submenu_page('dtt_task_manager_view_task',__( 'View Report', DTT_TEXT_DOMAIN ), __( 'View Report', DTT_TEXT_DOMAIN ), 'manage_options', 'dtt_task_manager_transactions',array($this, 'dtt_task_manager_transactions'));

					add_submenu_page('dtt_task_manager_view_task', __( 'Getting Started', DTT_TEXT_DOMAIN ),__( 'Getting Started', DTT_TEXT_DOMAIN ), 'manage_options', 'dtt_task_intro',array($this, 'dtt_task_intro'));

					add_submenu_page('dtt_task_manager_view_task', __( 'Support', DTT_TEXT_DOMAIN ),__( 'Support', DTT_TEXT_DOMAIN ), 'manage_options', 'https://dothattask.com/support/');
					

					add_submenu_page('',__( 'View Task Board', DTT_TEXT_DOMAIN ), __( 'View Task Board', DTT_TEXT_DOMAIN ), 'manage_options', 'dtt_task_manager_board',array($this, 'dtt_task_manager_board'));	

					add_submenu_page('',__( 'DoThatTask', DTT_TEXT_DOMAIN ), sprintf(__( 'DoThatTask %s', DTT_TEXT_DOMAIN ),__('Upgrade', DTT_TEXT_DOMAIN )), 'manage_options', 'dtt_plugin_upgrade',array($this, 'dtt_plugin_upgrade'));
					$page = sanitize_text_field(isset($_REQUEST['page'])) ? sanitize_text_field($_REQUEST['page']) : '';
					/*if(!empty($page) && in_array($page,['dtt_plugin_details','dtt_plugin_upgrade'])){
						exit( wp_redirect('admin.php?page=dtt_task_manager_view_task') );
					}*/
				}
			}
		}
		/**
	     * Plugin Page.
	     * 
	     * Used to call plugin custom pages.
	     *
	     * @access  public
	     * @param   mixed[] 	$aParams 	an array 
	     * @return  none
	     */
		public function dtt_task_manager_view_task($aParams)
		{
			$this->set_template('view',$aParams,'html','All tasks');
		}
		public function dtt_task_manager_transactions($aParams)
		{
			$this->set_template('transaction',$aParams,'html','Transaction History');
		}
		public function dtt_task_manager_settings($aParams)
		{
			$this->set_template('index',$aParams,'html','My account');
		}
		public function dtt_task_manager_footer($aParams)
		{ 
			echo $this->set_template('footer_logo',$aParams,"nohtml");
		}	
		public function dtt_task_manager_admin_footer($aParams)
		{ 
			$this->set_template('footer_admin',$aParams);
		}
		public function dtt_task_manager_board($aParams)
		{ 
			$this->set_template('task_board',$aParams,"html",__("Task Listing Board"));
		}
		public function dtt_plugin_details($aParams)
		{
			$this->set_template('plugin_details',$aParams,"html",__("Provide Valid"));	
		}
		public function dtt_plugin_upgrade($aParams)
		{
			$this->set_template('plugin_upgrade',$aParams,"html",__("Provide Valid"));
		}
		public function dtt_task_intro($aParams)
		{
			$this->set_template('intro',$aParams,'html',__('Getting Started'));
		}
		/**
	     * Plugin Css/Js Add.
	     * 
	     * Used to add css and js of plugin.
	     *
	     * @access  public
	     * @param   none
	     * @return  none
	     */
		public function dtt_enqueue_scripts($admin=0)
		{ 
			wp_enqueue_script('media-upload');
			wp_enqueue_style('dtt-task-wp-css',DTT_API_URL.'wp/css/front.css',array(),true);
		}
		/**
	     * Plugin Css/Js Add for admin panel.
	     * 
	     * Used to add css and js of plugin.
	     *
	     * @access  public
	     * @param   none
	     * @return  none
	     */
		public function dtt_admin_enqueue_scripts()
		{ 
			$wpVals = $this->functionClass->wpDefaultValues();
			$this->dtt_enqueue_scripts();
			
			wp_enqueue_style('dtt-task-style-font', 'https://maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css'  , array(), true);

			$page = sanitize_text_field(isset($_REQUEST['page'])) ? sanitize_text_field($_REQUEST['page']) : '';
			if(!empty($page) && in_array($page,$this->_page)){
				wp_enqueue_style('dtt-task-bootstrap-min',DTT_API_URL.'css/bootstrap.min.css',array(),true);
			}
			wp_enqueue_style('dtt-task-admin-css',DTT_API_URL.'wp/css/admin.css',array(),true);
			wp_enqueue_script('dtt-task-script',DTT_API_URL.'wp/js/admin.js',array(),true);
			wp_localize_script('dtt-task-script','task_ajax_object',array('url'=>admin_url('admin-ajax.php')));
			$translation_array = array('baseUrl'=>Common_Functions::BASEURL,'addClass'=>false,'pluginAssets'=>DTT_ASSETS);
			$translation_array['Installation_placeholder'] = __( 'Enter Valid Email', DTT_TEXT_DOMAIN );
			$translation_array['fabric_color'] = DTT_DEFAULT_COLOR_PRESET;
			//Audit Variable for JS
			$translation_array['audit_token'] = $this->functionClass->_auditToken;
			$translation_array['addtask_btn_for_audit'] = $this->functionClass->_addTaskBtnForAudit?1:0;
			$translation_array['domain'] = DTT_DOMAIN;
			$translation_array['app_url'] = DTT_API_URL;
			$translation_array['assets_url'] = DTT_API_URL.'wp/';
			$translation_array['task_plugin'] = 1;
			//localoze script variable for use in JS
			wp_localize_script( 'dtt-task-script', 'do_that_task_js_object', $translation_array );

			if(get_option(DTT_PLUGIN_ACTIVATE,false)){
				wp_enqueue_style('dtt-task-admin-popup',DTT_API_URL.'wp/css/popup.css',array(),true);
				if(!empty($page) && in_array($page,$this->_page)){
					wp_enqueue_script('dtt-task-moment-js',DTT_API_URL . 'js/moment/moment.min.js'   , array(), true,true);
					if(!empty($page) && $page=='dtt_task_manager_transactions'){
						wp_enqueue_script('dtt-task-daterangepicker-js',DTT_API_URL . 'js/daterangepicker/daterangepicker.min.js'   , array(), true,true);
						wp_enqueue_style('dtt-task-daterangepicker-css', DTT_API_URL . 'css/daterangepicker/daterangepicker.css' , array(), true);
					}
					wp_enqueue_style('dtt-select2-css', DTT_API_URL . 'css/select2/select2.min.css' , array(), true);
					wp_enqueue_script('select2-library-js',DTT_API_URL . 'js/select2/select2.full.min.js'   , array(), true,true);
					wp_localize_script( 'dtt-task-script', 'do_that_task_js_object_admin', ['addClass'=>true] );
				}
				//https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.5/js/select2.full.min.js
				if(!empty($page) && $page=='dtt_task_manager_view_task'){
					wp_enqueue_style('dtt-task-video-player-css', 'https://cdn.jsdelivr.net/npm/video.js@6.6.2/dist/video-js.css' , array(), true);
					wp_enqueue_style('dtt-task-video-player-theme-css', DTT_API_URL . 'wp/css/vue/custom-theme.css' , array(), true);
					wp_enqueue_script('vue-video-library-js',DTT_API_URL . 'wp/js/vue/video.min.js'   , array(), true,true);
					wp_enqueue_script('vue-video-player-library-js', DTT_API_URL . 'wp/js/vue/vue-video-player.js'   , array(), true,true);
					wp_enqueue_script('vue-library-js', DTT_API_URL . 'wp/js/vue.js'   , array(), true,true);
					wp_enqueue_script('vue-image-js',DTT_API_URL . 'wp/js/vue/v-img.min.js'   , array(), true,true);
					wp_enqueue_script('vue-paypal-checkout',DTT_API_URL . 'wp/js/vue/vue-paypal-checkout.min.js'   , array(), true,true);
					wp_enqueue_style('dtt-task-jkanban', DTT_API_URL . 'css/jkanban/jkanban.min.css' , array(), true);
					wp_enqueue_script('dtt-task-jkanban',DTT_API_URL . 'js/jkanban/jkanban.js'   , array(), false,true);
					wp_enqueue_script('vue-component',DTT_API_URL . 'wp/js/vue/index.js'   , array(), false,true);
					// Localize the script with new data
					$aDttSetting = get_option(DTT_SETTING);
					$translation_array = array('baseUrl'=>Common_Functions::BASEURL,
											'assetUrl'=>DTT_ASSETS_URL,
											'uploadedAssetUrl'=>DTT_UPLOADED_ASSETS_URL,
											'currentUData'=>json_encode($wpVals),
											'version' => DTT_VERSION,
											'upgradeUrl' => admin_url('admin.php?page=dtt_plugin_upgrade'),
											'pluginUrl'=>plugins_url( 'src/', dirname(__FILE__) ),
											'defaultTeam'=>isset($aDttSetting['default_team'])?$aDttSetting['default_team']:'',
											'subscription'=>(isset($aDttSetting['subscription']) && !empty(isset($aDttSetting['subscription']))?true:false),
											'assignAgency'=>isset($aDttSetting['agency_ref_id'])?true:false);
					wp_localize_script( 'vue-component', 'dtt_vue_js_object', $translation_array );
				}
			}
		}
		public function checkTokenExistOrNot(){
			$curlUrl = $this->functionClass->webServiceUrl("check_audit");
			$wpVals = $this->functionClass->wpDefaultValues();
			$aPostData = array("token" => $this->functionClass->_auditToken);
			$this->functionClass->__defaultMethod = "POST";
			$aResponse = $this->functionClass->postUrlUsingCurl($curlUrl,$aPostData);
			$response = json_decode($aResponse,true);
			if(isset($response['code']) && $response['code']==1){
				return true;
			}else{
				if(isset($_SESSION['dtt_audit_token'])){
					unset($_SESSION['dtt_audit_token']);
					unset($_SESSION['dtt_audit_token_expire']);
				}
			}
			return false;
		} 
		/**
	     * Plugin Admin Bar.
	     * 
	     * Used to customize plugin toolbar at admin panel.
	     *
	     * @access  public
	     * @param   obj 	$wp_admin_bar 	An instance of the global object WP_Admin_Bar
	     * @return  none
	     */
		public function dtt_task_manager_modify_admin_bar($wp_admin_bar){
			global $wp;
			$args = array(
			  'id'    => 'my_page',
			  'title' => '<img src="'.$this->_doThatTaskIcon.'" alt="Plugin Icon" style="vertical-align:middle;margin-right:5px;"> '.__( 'Task', DTT_TEXT_DOMAIN ),
			  'parent' => "new-content",
			  'href'  => '#',
			  'meta'  => array( 'class' => '','data-action'=>'' ),
			  
			 );
			 $wp_admin_bar->add_node( $args );
		}
		/**
	     * Set Template.
	     * 
	     * Used to set teamplate file on page.
	     *
	     * @access  public
	     * @param   string 		$tempName	template name which we want to set on page.
	     * @param   mixed[]		$aOpts 		an array params which we get on page.
	     * @param   string 		$view 		view type.
	     * @param   string 		$title 		template title.
	     * @return  string 		$html 		html content which we get form template.
	     */
		public function set_template($tempName,$aOpts,$view = '',$title = '')
		{		
			global $dttTaskFunObj;
			$aParams['template'] = $tempName;
			$aParams['view'] = $view;
			$aParams['title'] = $title;
			$aVars = $aOpts;// ? array_merge($this->aParams,$aOpts) : $this->aParams;
			ob_start();	
			include DTT_PATH."template/template.php";		
			if($view=='nohtml'){
				return ob_get_clean();	
			}else{
				return ob_get_contents();		
				ob_get_clean();
			}
		}
		/**
	     * .DTT plugin redirect after just plugin activate
	     * 
	     * @access  public
	     * @param   none
	     * @return  none
	     */
		public function dtt_plugin_redirect(){
			global $dttTaskFunObj;
			if (get_option(DTT_DO_ACTIVATION_REDIRECT, false)) {
				$dttTaskFunObj->addRole();
				add_option(DTT_DO_INSTALLATION, true);
		        $dttTaskFunObj->deleteOption([DTT_DO_ACTIVATION_REDIRECT]);
		        exit( wp_redirect('admin.php?page=dtt_plugin_details') );
		    }
		}
		/**
	     * Activate DoThatTask plugin.
	     * 
	     * Used to activatae that plugin and call web-service to create user at App end.
	     *
	     * @access  public
	     * @param   none
	     * @return  none
	     */
		public function activation_dtt_plugin(){
			add_option(DTT_DO_ACTIVATION_REDIRECT, true);
		}
		

		/**
	     * DeActivate DoThatTask plugin.
	     * 
	     * Used to deactivate that plugin.
	     *
	     * @access  public
	     * @param   none
	     * @return  none
	     */
		public function deactivation_dtt_plugin(){
			global $dttTaskFunObj;
			//wp_delete_user(3);
			$dttTaskFunObj->pluginDeactive();
			$dttTaskFunObj->pluginUninstall();
		}

		/**
	     * UnInstall DoThatTask plugin.
	     * 
	     * Used to delete plugin varibale which was created for plugin uses.
	     *
	     * @access  public
	     * @param   none
	     * @return  none
	     */
		public function uninstall_dtt_plugin(){
			//die("asdasdadasdadad uninstal");
		}		
	}
}