<?php
namespace DoThatTask\Includes;

if ( ! defined( 'ABSPATH' ) ) {
	die("You can't access this file directly"); // disable direct access
} 

if ( ! class_exists( 'Dtt_Ajax' ) ) {

	class Dtt_Ajax
	{		
		protected static $_instance = null;
		/**
		 * Main DoThatTask Instance.
		 *
		 * @since 
		 * @static
		 * @see init()
		 * @return DoThatTask - Main instance.
		 */
		public static function init() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		public function __construct()
		{
			//This funcation init all hooks which required for plugin.
			$this->init_hooks();
		}
		protected function init_hooks(){
			//Add task by this ajax funcation
			add_action( 'wp_ajax_dtt_task_manager_add_ajax', array(&$this,'dtt_task_manager_add_ajax' ));
			add_action('wp_ajax_nopriv_dtt_task_manager_add_ajax', array(&$this,'dtt_task_manager_add_ajax'));//this hook use for non-authenticated 

			//Add Recordings options by this ajax funcation
			add_action( 'wp_ajax_dtt_recording_manager', array(&$this,'dtt_recording_manager_add_ajax' ));
			add_action('wp_ajax_nopriv_dtt_recording_manager', array(&$this,'dtt_recording_manager_add_ajax'));//this hook use for non-authenticated 

			//Add review for plugin by this ajax funcation
			add_action( 'wp_ajax_dtt_add_review_ajax', array(&$this,'dtt_add_review' ));
			add_action('wp_ajax_nopriv_dtt_add_review_ajax', array(&$this,'dtt_add_review'));//this hook use for non-authenticated 

			//Upload image for task byt this ajax funcation
			add_action( 'wp_ajax_dtt_task_upload_image', array(&$this,'dtt_task_upload_image_ajax' ));
			add_action('wp_ajax_nopriv_dtt_task_upload_image', array(&$this,'dtt_task_upload_image_ajax'));//this hook use for non-authenticated 

			//Show popup for add task HTML , ajax
			add_action( 'wp_ajax_dtt_task_popup', array(&$this,'dtt_task_popup' ));
			add_action('wp_ajax_nopriv_dtt_task_popup', array(&$this,'dtt_task_popup'));//this hook use for non-authenticated 

			//Create Admin user for task
			add_action( 'wp_ajax_dtt_task_user_create', array(&$this,'dtt_task_user_create' ));
			add_action('wp_ajax_nopriv_dtt_task_user_create', array(&$this,'dtt_task_user_create'));//this hook use for non-authenticated 
			//Create Admin user for task
			add_action( 'wp_ajax_dtt_task_user_remove', array(&$this,'dtt_task_user_remove' ));
			add_action('wp_ajax_nopriv_dtt_task_user_remove', array(&$this,'dtt_task_user_remove'));//this hook use for non-authenticated 
			//Place comment on task
			add_action( 'wp_ajax_dtt_task_comment', array(&$this,'dtt_task_comment' ));
			//add_action('wp_ajax_nopriv_dtt_task_user_remove', array(&$this,'dtt_task_user_remove'));//this hook use for non-authenticated 
		}
		/**
	     * Add Review for that plugin.
	     * 
	     * Used to add review for that plugin , about their experiences.
	     *
	     * @access  public
	     * @param   none
	     * @return  string $aResponse  string consist json array.
	     */
		public function dtt_add_review(){
			global $dttTaskFunObj;
			$addReview = sanitize_text_field(isset($_POST['dtt_add_review_nonce_field'])) ? sanitize_text_field($_POST['dtt_add_review_nonce_field']) : '';
			if(( !empty($addReview)  && wp_verify_nonce( $addReview, 'dtt_add_review_nonce_action' ) ))
			{
				$aVals =  array_map( 'sanitize_text_field', wp_unslash( $_POST['val'] ) );	
				
				$curlUrl = $dttTaskFunObj->webServiceUrl("add_review");
				$wpVals = $dttTaskFunObj->wpDefaultValues();
				$aDttSetting = get_option(DTT_SETTING);
				$aPostData = array("email" => $wpVals['admin_email'],'review'=>$aVals['review']);
				
				$dttTaskFunObj->__defaultMethod = "POST";
				$aResponse = $dttTaskFunObj->postUrlUsingCurl($curlUrl,$aPostData,['AccessToken'=>$wpVals['admin_token']]);
				//$response = json_decode($aResponse,true);
			}else{
				$aResponse = json_encode(array('code'=>0,'message'=>__('There is a problem in your request',DTT_TEXT_DOMAIN)));
			}	
			echo $aResponse;
			exit;
		}
		/**
	     * Add comment in the task.
	     * 
	     * send message to all associate experts
	     *
	     * @access  public
	     * @param   none
	     * @return  string $aResponse  string consist json array.
	     */
		public function dtt_task_comment(){
			global $dttTaskFunObj;
			if(sanitize_text_field(isset($_POST['action'])) && sanitize_text_field(($_POST['action']))=='dtt_task_comment')
			{
				$upload_dir  = wp_upload_dir();
				$baseUrl = $upload_dir['baseurl'].'/'.'dothattask'.'/';
				$upload_path = trailingslashit( $upload_dir['basedir'] ).'dothattask'.'/' ;

				$uploadedFile = [];
				if(sanitize_text_field(isset($_FILES['file'])) && count($_FILES['file'])>0){
					list($uploadedFile,$extension,$error) = $this->dtt_task_upload_files();	
				}
				$aVals =  array_map( 'sanitize_text_field', wp_unslash( $_POST ) );	
				$aVals['message'] = sanitize_textarea_field(wp_unslash($_POST['message']));

				$wpVals = $dttTaskFunObj->wpDefaultValues();
				$curlUrl = $dttTaskFunObj->webServiceUrl("task/send_message/".$wpVals['admin_email'].'/'.$aVals['tId']);
				$aDttSetting = get_option(DTT_SETTING);
				$aPostData = array("message" => $aVals['message'],'files'=>$uploadedFile);
				
				$dttTaskFunObj->__defaultMethod = "POST";
				
				$aResponse = $dttTaskFunObj->postUrlUsingCurl($curlUrl,$aPostData,['AccessToken'=>$wpVals['admin_token']]);
				
				//Unlink all images
				foreach ($uploadedFile as $key => $value) {
					unlink(str_replace($baseUrl, $upload_path, $value));
				}
				//$response = json_decode($aResponse,true);
				
			}else{
				$aResponse = json_encode(array('code'=>0,'message'=>__('There is a problem in your request',DTT_TEXT_DOMAIN)));
			}	
			echo $aResponse;
			exit;
		}
		/**
		 * Add Recordings Request.
	     * 
	     * Used to add task data at APP end.
	     *
	     * @access  public
	     * @param   none
	     * @return  string $aResponse  string consist json array.
	     */
		public function dtt_recording_manager_add_ajax()
		{
			if (!empty($_POST)) {
				$prefix = "user_";
				$upload_dir  = wp_upload_dir();
				
				$baseUrl = $upload_dir['baseurl'].'/'.'dothattask';
				$upload_path = trailingslashit( wp_upload_dir()['basedir'] ).'dothattask'.'/' ;

				$aFile['name'] =  $_POST['name'];	
				$aFile['type'] =  ( $_FILES['file']['type'] );	
				$aFile['tmp_name'] = ( $_FILES['file']['tmp_name'] );	
				$aFile['error'] =  wp_unslash( $_FILES['file']['error'] );	
				$aFile['size'] =  wp_unslash( $_FILES['file']['size'] );	
				$fileUploaded = '';
				$extension = '';
				$error = "";
				foreach($aFile as $key=>$file){
					if($aFile['type']=='audio/webm' || $aFile['type']=='audio/ogg' || $aFile['type']=='audio/wav'){
						$hashed_filename = $prefix.md5( microtime() ).'_'.time().'.'.'mp3';
						$ext = 'mp3';
					}else{
						$hashed_filename = $prefix.md5( microtime() ).'_'.time().'.'.pathinfo($file,PATHINFO_EXTENSION);
						$ext = pathinfo($file,PATHINFO_EXTENSION);
					}
					if(!move_uploaded_file($aFile['tmp_name'],$upload_path . $hashed_filename)){
						$error .= '1';
					}else{
						$fileUploaded = $baseUrl.'/'.$hashed_filename;
						$extension = strtolower($ext);
					}
				}
				echo json_encode(array('uploads'=> $fileUploaded,'extension'=>$extension, 'message'=>'Data get successfully'));
			}else{
				echo json_encode(array('error'=>1, 'message'=>'Data not found'));
			}
			exit;
		}
		/**
	     * Task Add Request.
	     * 
	     * Used to add task data at APP end.
	     *
	     * @access  public
	     * @param   none
	     * @return  string $aResponse  string consist json array.
	     */
		public function dtt_task_manager_add_ajax()
		{
			global $dttTaskFunObj;
			$addTask = sanitize_text_field(isset($_POST['dtt_addtask_nonce_field'])) ? sanitize_text_field($_POST['dtt_addtask_nonce_field']) : '';
			if(( !empty($addTask)  && wp_verify_nonce( $addTask, 'dtt_addtask_nonce_action' ) ))
			{
				$upload_dir  = wp_upload_dir();
				$baseUrl = $upload_dir['baseurl'].'/'.'dothattask'.'/';
				$upload_path = trailingslashit( $upload_dir['basedir'] ).'dothattask'.'/' ;
				//$aVals = $_POST;
				$aVals =  array_map( 'sanitize_text_field', wp_unslash( $_POST['val'] ) );	

				$aVals['task_type'] = sanitize_text_field(isset($_POST['task_type']))? sanitize_text_field($_POST['task_type']):'';
				
				if(isset($aVals['upload_file']) && !empty($aVals['upload_file'])){
					$aVals['upload_file'] = explode(',', $aVals['upload_file']);
				}
				if(isset($aVals['remove_file']) && !empty($aVals['remove_file'])){
					$aVals['remove_file'] = explode(',', $aVals['remove_file']);
					foreach ($aVals['remove_file'] as $remove) {
						if(isset($aVals['upload_file'][$remove])){
							//echo str_replace($baseUrl, $upload_path, $aVals['upload_file'][$remove]);
							unlink(str_replace($baseUrl, $upload_path, $aVals['upload_file'][$remove]));
							unset($aVals['upload_file'][$remove]);
						}
					}
					//$aVals['upload_file'] = implode(',', array_filter($aVals['upload_file']));
				}
				unset($aVals['remove_file']);
				$curlUrl = $dttTaskFunObj->webServiceUrl("tasks/create");
				$wpVals = $dttTaskFunObj->wpDefaultValues();
				$aDttSetting = get_option(DTT_SETTING);
				$aPostData = array("email" => $wpVals['admin_email'],"name" =>$aVals['task_title'],"url" =>$aVals['task_url'],
					"description" =>$aVals['task_description'],'image'=>$aVals['upload_file'],'priority'=>$aVals['task_priority'],'team'=>isset($aDttSetting['default_team'])?$aDttSetting['default_team']:'','task_type'=>$aVals['task_type'],'reporter'=>'');
				if($dttTaskFunObj->_addTaskBtnForAudit){
					$aPostData['audit_token'] = $dttTaskFunObj->_auditToken;
					$aPostData['reporter'] = 3;
				}else{
					if(isset($wpVals['user_role']) && $wpVals['user_role']=='administrator'){
						$aPostData['reporter'] = 1;
					}else{
						$aPostData['reporter'] = 2;
					}
				}
				$dttTaskFunObj->__defaultMethod = "POST";

				$aResponse = $dttTaskFunObj->postUrlUsingCurl($curlUrl,$aPostData,['AccessToken'=>$wpVals['admin_token']]);
				
				$response = json_decode($aResponse,true);
				
				//Unlink all images
				//if($response['code']==1){
					/*$prefix = "user_".get_current_user_id()."_";
					$upload_dir  = wp_upload_dir();
					$upload_path = str_replace( '/', DIRECTORY_SEPARATOR, $upload_dir['path'] ) . DIRECTORY_SEPARATOR;
					$mask = $upload_path.$prefix.'*.*';*/
					//array_map('unlink', glob($mask));
					foreach ($aVals['upload_file'] as $key => $value) {
						unlink(str_replace($baseUrl, $upload_path, $value));
					}
				//}
			}else{
				$aResponse = json_encode(array('code'=>0,'message'=>__('There is a problem in your request',DTT_TEXT_DOMAIN)));
			}	
			echo $aResponse;
			exit;
		}
		/**
	     * Task Upload Image
	     * 
	     * Used to upload task image by ajax request.
	     *
	     * @access  public
	     * @param   none
	     * @return  string $response  string consist json array.
	     */
		public function dtt_task_upload_image_ajax()
		{
			WP_Filesystem($_SESSION['creds']);

	        global $wp_filesystem;
			//global $wp_filesystem;
			$prefix = "user_".get_current_user_id()."_";
			$upload_dir  = wp_upload_dir();
			$baseUrl = $upload_dir['baseurl'].'/'.'dothattask';
			$upload_path = trailingslashit( wp_upload_dir()['basedir'] ).'dothattask'.'/' ;

			if (! is_dir($upload_path)) {
		       wp_mkdir_p( $upload_path );
		    }
		    $aRatio = ['width'=>'','height'=>''];
			$aVals =  array_map( 'sanitize_text_field', wp_unslash( $_POST ) );	
			$hashed_filename = '';
			if(isset($aVals['base64data']) && !empty($aVals['base64data'])){
				
				$aVals['type'] = 'image';
				$data = $aVals['base64data'];
				list($type, $data) = explode(';', $data);
				list(, $data)      = explode(',', $data);
				$data = base64_decode($data);
				$fileName = time().'.png';
				$aRatio = ['width'=>$aVals['width'],'height'=>$aVals['height']];
				
				$hashed_filename = $prefix.md5( microtime() ) . '_' . $fileName;
				if(isset($aVals['file_name']) && !empty($aVals['file_name'])){
					$mask = $upload_path.$aVals['file_name'];
					unlink($mask);
					//array_map('unlink', glob($mask));
				}

				// Save the image in the uploads directory.
				$upload_file = file_put_contents( $upload_path . $hashed_filename, $data );
				if(!$upload_file){
					echo json_encode(array());
					exit;
				}
				$capture = $hashed_filename;
				$hashed_filename = $baseUrl.'/'.$hashed_filename;
				//print_r($rData);
			}else{
				//Image upload 
				list($uploadedFile,$extension,$error) = $this->dtt_task_upload_files();
			}

			if($aVals['type']!='files' && (!empty($uploadedFile) || !empty($hashed_filename))){
				if(empty($hashed_filename)){
					$hashed_filename = $uploadedFile[0];
					$capture = str_replace($baseUrl.'/', '', $hashed_filename);
				}
				list($origWidth, $origHeight) = getimagesize($hashed_filename);
				$widthRatio = $aRatio['width'] / $origWidth;
			    $heightRatio = $aRatio['height'] / $origHeight;

			    // Ratio used for calculating new image dimensions.
			    $ratio = min($widthRatio, $heightRatio);

			    // Calculate new image dimensions.
			    $attr = ['origWidth'=>$origWidth,'origHeight'=>$origHeight];
			    $attr['width']  = (int)$origWidth * $ratio;
			    if($origWidth<$aRatio['width']){
			    	$attr['width']  = (int)$origWidth;
			    }
			    $attr['height'] = (int)$origHeight * $ratio;
			    if($origHeight<$aRatio['height']){
			    	$attr['height'] = (int)$origHeight;
			    }
				echo json_encode(array('image'=> $hashed_filename,'fileName'=>$hashed_filename,'attr'=>$attr,'capture'=>(isset($capture)?$capture:'')));
			}else{
				echo json_encode(array('uploads'=> $uploadedFile,'extension'=>$extension,'error'=>$error));
			}
			exit;
		}

		public function dtt_task_upload_files()
		{
			$prefix = "user_";
			$upload_dir  = wp_upload_dir();
			
			$baseUrl = $upload_dir['baseurl'].'/'.'dothattask';
			$upload_path = trailingslashit( wp_upload_dir()['basedir'] ).'dothattask'.'/' ;

			$aFile['name'] =  array_map( 'sanitize_file_name', wp_unslash( $_FILES['file']['name'] ) );	
			$aFile['type'] =  ( $_FILES['file']['type'] );	
			$aFile['tmp_name'] = ( $_FILES['file']['tmp_name'] );	
			$aFile['error'] =  wp_unslash( $_FILES['file']['error'] );	
			$aFile['size'] =  wp_unslash( $_FILES['file']['size'] );	
			
			$fileUploaded = [];
			$extension = [];
			$error = "";
			foreach($aFile['name'] as $key=>$file){
				$ext = pathinfo($file,PATHINFO_EXTENSION);
				$hashed_filename = $prefix.md5( microtime() ).'_'.time().'.'.pathinfo($file,PATHINFO_EXTENSION);
				if(!move_uploaded_file($aFile['tmp_name'][$key],$upload_path . $hashed_filename)){
					$error .= ($key+1).',';
				}else{
					$fileUploaded[] = $baseUrl.'/'.$hashed_filename;
					$extension[] = strtolower($ext);
				}
			}
			return [$fileUploaded,$extension,$error];
			
		}
		/**
	     * Task PopUp
	     * 
	     * Used to open add task form in popup.
	     *
	     * @access  public
	     * @param   none
	     * @return  string $response  string consist json array.
	     */
		public function dtt_task_popup(){
			global $dttTaskFunObj;
			ob_start();	

			$id = "dtt-admin-task-popup";
			$link = sanitize_text_field($_POST['link']);

			if (get_option(DTT_PLUGIN_UPGRADE, false)) {
				$dttTaskFunObj->jsonEncode(array('html'=>'','id'=>$id,'redirect'=>admin_url('admin.php?page=dtt_plugin_upgrade')));
			}
			if($link=='add-review'){
				include DTT_PATH."template/review_popup.php";
			}else{
				$curlViewUrl = $dttTaskFunObj->webServiceUrl("wp_plugin_setting/priority");
				$dttTaskFunObj->__defaultMethod = 'GET';
				$aResponse = $dttTaskFunObj->postUrlUsingCurl($curlViewUrl);
				$aJson = json_decode($aResponse,true);
				$priority = array_reverse($aJson['priorities']['options']);
				$defaultPriority = $aJson['priorities']['default'];
				$alreadyActive = 1;
				if($_POST['alreadyActive']=='false'){
					$alreadyActive = 0;
				}
				$currentUser = $dttTaskFunObj->wpDefaultValues();
				include DTT_PATH."template/ajax_popup_content.php";
			}
			$temp = ob_get_contents();		
			ob_get_clean();
			$dttTaskFunObj->jsonEncode(array('html'=>$temp,'id'=>$id));
		}
		/**
	     * Task User Create
	     * 
	     * Used to create admin user for the task.
	     *
	     * @access  public
	     * @param   none
	     * @return  string $response  string consist json array.
	     */
		public function dtt_task_user_create(){

			global $dttTaskFunObj;
			try{
				$aVals =  array_map( 'sanitize_text_field', wp_unslash( $_POST ) );	
				$response = ['code'=>0];
				$keepData = isset($aVals['keepData'])?$aVals['keepData']:1;
				if(isset($aVals['access']) && isset($aVals['tId']) && !empty($aVals['tId'])){
					$uName = DTT_DOMAIN.'_'.$aVals['tId'];
					if($aVals['access']==0 || $aVals['access']==2){
						$userDetail = ['username'=>$uName,'email'=>$uName.'@'.DTT_APP_DOMAIN,'password'=>$dttTaskFunObj->randomPassword()];
						$response = $dttTaskFunObj->addTaskUser($userDetail);
						$response['access'] = 1;
					}elseif($aVals['access']==1){
						$response = $dttTaskFunObj->removeTaskUser($uName,$keepData);
						$response['access'] = 2;
					}
					if($response['code']==1){
						$curlUrl = $dttTaskFunObj->webServiceUrl("update_task_access");
						$wpVals = $dttTaskFunObj->wpDefaultValues();
						$aPostData = array("task_id" => $aVals['tId'],"access" =>$response['access']);	
						$response['access_detail'] = '';
						if(isset($userDetail)){
							$response['access_detail'] = $aPostData['task_user'] = $userDetail;
							$aPostData['task_user']['url'] = home_url();
						}					
						$dttTaskFunObj->__defaultMethod = "POST";

						$aResponse = $dttTaskFunObj->postUrlUsingCurl($curlUrl,$aPostData,['AccessToken'=>$wpVals['admin_token']]);
						
						$result = json_decode($aResponse,true);
						if(!isset($result['code']) || $result['code']==0){
							$response['code'] = 0;
							if($response['access']==1){
								$res = $dttTaskFunObj->removeTaskUser($uName,$keepData);
							}
							throw new \Exception("There is a problem in your request. Try Again", 1);
						}
						//$response['email'] = $result['email'];
					}else{
						if($response['access']==1){
							$res = $dttTaskFunObj->removeTaskUser($uName,$keepData);
						}
						throw new \Exception("There is a problem in your request. Try Again", 1);
					}
				}else{
					throw new \Exception("There is a problem in your request.", 1);
				}
			}catch(\Exception $e){
				$response['message'] = $e->getMessage();
			}
			$dttTaskFunObj->jsonEncode($response);
		}
		/**
	     * Task User Create
	     * 
	     * Used to create admin user for the task.
	     *
	     * @access  public
	     * @param   none
	     * @return  string $response  string consist json array.
	     */
		public function dtt_task_user_remove(){

			global $dttTaskFunObj;
			try{
				$aVals =  array_map( 'sanitize_text_field', wp_unslash( $_POST ) );	
				$response = ['code'=>0];
				if(isset($aVals['tId']) && !empty($aVals['tId'])){
					$uName = DTT_DOMAIN.'_'.$aVals['tId'];
					$response = $dttTaskFunObj->removeTaskUser($uName);
				}else{
					throw new \Exception("There is a problem in your request.", 1);
				}
			}catch(\Exception $e){
				$response['message'] = $e->getMessage();
			}
			$dttTaskFunObj->jsonEncode($response);
		}	
	}
}